@@ -26,6 +26,47 @@ extern int verbose;
 
 static struct exclude_struct **exclude_list;
 
+/*
+ * Optimization for special case when all included files are explicitly
+ *   listed without wildcards in the "exclude" list followed by a "- *"
+ *   to exclude the rest.
+ * Contributed by Dave Dykstra <dwd@bell-labs.com>
+ */
+static int only_included_files = 1;
+static struct exclude_struct *exclude_the_rest;
+
+int send_included_file_names(int f,struct file_list *flist)
+{
+	struct exclude_struct *ex, **ex_list;
+	int n;
+	char *p;
+
+	if (!only_included_files || (exclude_the_rest == NULL))
+		return 0;
+
+	if (verbose > 1) {
+		rprintf(FINFO,"(using include-only optimization) ");
+	}
+
+	/* set exclude_list to NULL temporarily so check_exclude */
+	/*   will always return true */
+	ex_list = exclude_list;
+	exclude_list = NULL;
+	for (n=0; (ex = ex_list[n]) != NULL; n++) {
+		if (ex == exclude_the_rest)
+			break;
+		p = ex->pattern;
+		while (*p == '/') {
+			/* skip the allowed beginning slashes */
+			p++;
+		}
+		send_file_name(f,flist,p,0,0);
+	}
+	exclude_list = ex_list;
+	
+	return 1;
+}
+
 /* build an exclude structure given a exclude pattern */
 static struct exclude_struct *make_exclude(char *pattern, int include)
 {
@@ -36,8 +77,6 @@ static struct exclude_struct *make_exclude(char *pattern, int include)
 
 	memset(ret, 0, sizeof(*ret));
 
-	ret->orig = strdup(pattern);
-
 	if (strncmp(pattern,"- ",2) == 0) {
 		pattern += 2;
 	} else if (strncmp(pattern,"+ ",2) == 0) {
@@ -49,9 +88,18 @@ static struct exclude_struct *make_exclude(char *pattern, int include)
 
 	ret->pattern = strdup(pattern);
 
-	if (!ret->orig || !ret->pattern) out_of_memory("make_exclude");
+	if (!ret->pattern) out_of_memory("make_exclude");
 
-	if (strpbrk(pattern, "*[?")) ret->regular_exp = 1;
+	if (strpbrk(pattern, "*[?")) {
+	    if (!ret->include && (*pattern == '*') && (*(pattern+1) == '\0')) {
+		    exclude_the_rest = ret;
+	    } else {
+		    only_included_files = 0;
+	    }
+	    ret->regular_exp = 1;
+	} else if (!ret->include) {
+		only_included_files = 0;
+	}
 
 	if (strlen(pattern) > 1 && pattern[strlen(pattern)-1] == '/') {
 		ret->pattern[strlen(pattern)-1] = 0;
@@ -67,7 +115,6 @@ static struct exclude_struct *make_exclude(char *pattern, int include)
 
 static void free_exclude(struct exclude_struct *ex)
 {
-	free(ex->orig);
 	free(ex->pattern);
 	memset(ex,0,sizeof(*ex));
 	free(ex);
@@ -143,6 +190,8 @@ void add_exclude_list(char *pattern,struct exclude_struct ***list, int include)
 		}
 		free((*list));
 		*list = NULL;
+		only_included_files = 1;
+		exclude_the_rest = NULL;
 		return;
 	}
 
@@ -172,7 +221,7 @@ struct exclude_struct **make_exclude_list(char *fname,
 	if (!f) {
 		if (fatal) {
 			rprintf(FERROR,"%s : %s\n",fname,strerror(errno));
-			exit_cleanup(1);
+			exit_cleanup(RERR_FILEIO);
 		}
 		return list;
 	}
@@ -207,23 +256,21 @@ void send_exclude_list(int f)
 	}
 
 	for (i=0;exclude_list[i];i++) {
-		char *pattern = exclude_list[i]->orig; 
+		char *pattern = exclude_list[i]->pattern; 
 		int l;
 
-		if (remote_version < 19) {
-			if (strncmp(pattern,"+ ", 2)==0) {
-				rprintf(FERROR,"remote rsync does not support include syntax - aborting\n");
-				exit_cleanup(1);
-			}
-			
-			if (strncmp(pattern,"- ", 2) == 0) {
-				pattern += 2;
-			}
-		}
-		
 		l = strlen(pattern);
 		if (l == 0) continue;
-		write_int(f,l);
+		if (exclude_list[i]->include) {
+			if (remote_version < 19) {
+				rprintf(FERROR,"remote rsync does not support include syntax - aborting\n");
+				exit_cleanup(RERR_NOSUPPORT);
+			}
+			write_int(f,l+2);
+			write_buf(f,"+ ",2);
+		} else {
+			write_int(f,l);
+		}
 		write_buf(f,pattern,l);
 	}    
 
