@@ -51,6 +51,36 @@ static struct exclude_struct **local_exclude_list;
 
 static void clean_flist(struct file_list *flist, int strip_root);
 
+
+static void list_file_entry(struct file_struct *f)
+{
+	char perms[11] = "----------";
+	char *perm_map = "rwxrwxrwx";
+	int i;
+
+	for (i=0;i<9;i++) {
+		if (f->mode & (1<<i)) perms[9-i] = perm_map[8-i];
+	}
+	if (S_ISLNK(f->mode)) perms[0] = 'l';
+	if (S_ISDIR(f->mode)) perms[0] = 'd';
+	if (S_ISBLK(f->mode)) perms[0] = 'b';
+	if (S_ISCHR(f->mode)) perms[0] = 'c';
+	if (S_ISSOCK(f->mode)) perms[0] = 's';
+	if (S_ISFIFO(f->mode)) perms[0] = 'p';
+	
+	if (preserve_links && S_ISLNK(f->mode)) {
+		rprintf(FINFO,"%s %11.0f %s %s -> %s\n", 
+			perms, 
+			(double)f->length, timestring(f->modtime), 
+			f_name(f), f->link);
+	} else {
+		rprintf(FINFO,"%s %11.0f %s %s\n", 
+			perms, 
+			(double)f->length, timestring(f->modtime), f_name(f));
+	}
+}
+
+
 int link_stat(const char *Path, STRUCT_STAT *Buffer) 
 {
 #if SUPPORT_LINKS
@@ -89,6 +119,23 @@ static void set_filesystem(char *fname)
 }
 
 
+static int to_wire_mode(mode_t mode)
+{
+	if (S_ISLNK(mode) && (S_IFLNK != 0120000)) {
+		return (mode & ~(_S_IFMT)) | 0120000;
+	}
+	return (int)mode;
+}
+
+static mode_t from_wire_mode(int mode)
+{
+	if ((mode & (_S_IFMT)) == 0120000 && (S_IFLNK != 0120000)) {
+		return (mode & ~(_S_IFMT)) | S_IFLNK;
+	}
+	return (mode_t)mode;
+}
+
+
 static void send_directory(int f,struct file_list *flist,char *dir);
 
 static char *flist_dir;
@@ -147,7 +194,7 @@ static void send_file_entry(struct file_struct *file,int f,unsigned base_flags)
 	if (!(flags & SAME_TIME))
 		write_int(f,(int)file->modtime);
 	if (!(flags & SAME_MODE))
-		write_int(f,(int)file->mode);
+		write_int(f,to_wire_mode(file->mode));
 	if (preserve_uid && !(flags & SAME_UID)) {
 		add_uid(file->uid);
 		write_int(f,(int)file->uid);
@@ -248,7 +295,7 @@ static void receive_file_entry(struct file_struct **fptr,
 	file->flags = flags;
 	file->length = read_longint(f);
 	file->modtime = (flags & SAME_TIME) ? last_time : (time_t)read_int(f);
-	file->mode = (flags & SAME_MODE) ? last_mode : (mode_t)read_int(f);
+	file->mode = (flags & SAME_MODE) ? last_mode : from_wire_mode(read_int(f));
 	if (preserve_uid)
 		file->uid = (flags & SAME_UID) ? last_uid : (uid_t)read_int(f);
 	if (preserve_gid)
@@ -432,7 +479,7 @@ static struct file_struct *make_file(char *fname)
 
 
 
-static void send_file_name(int f,struct file_list *flist,char *fname,
+void send_file_name(int f,struct file_list *flist,char *fname,
 			   int recursive, unsigned base_flags)
 {
   struct file_struct *file;
@@ -532,7 +579,7 @@ struct file_list *send_file_list(int f,int argc,char *argv[])
 {
 	int i,l;
 	STRUCT_STAT st;
-	char *p,*dir;
+	char *p,*dir,*olddir;
 	char lastpath[MAXPATHLEN]="";
 	struct file_list *flist;
 	int64 start_write;
@@ -580,6 +627,7 @@ struct file_list *send_file_list(int f,int argc,char *argv[])
 		}
 
 		dir = NULL;
+		olddir = NULL;
 
 		if (!relative_paths) {
 			p = strrchr(fname,'/');
@@ -615,7 +663,7 @@ struct file_list *send_file_list(int f,int argc,char *argv[])
 			fname = ".";
 		
 		if (dir && *dir) {
-			char *olddir = push_dir(dir, 1);
+			olddir = push_dir(dir, 1);
 
 			if (!olddir) {
 				io_error=1;
@@ -625,21 +673,22 @@ struct file_list *send_file_list(int f,int argc,char *argv[])
 			}
 
 			flist_dir = dir;
-			if (one_file_system)
-				set_filesystem(fname);
-			send_file_name(f,flist,fname,recurse,FLAG_DELETE);
-			flist_dir = NULL;
-			if (pop_dir(olddir) != 0) {
-				rprintf(FERROR,"pop_dir %s : %s\n",
-					dir,strerror(errno));
-				exit_cleanup(1);
-			}
-			continue;
 		}
 		
 		if (one_file_system)
 			set_filesystem(fname);
-		send_file_name(f,flist,fname,recurse,FLAG_DELETE);
+
+		if (!recurse || !send_included_file_names(f,flist))
+			send_file_name(f,flist,fname,recurse,FLAG_DELETE);
+
+		if (olddir != NULL) {
+			flist_dir = NULL;
+			if (pop_dir(olddir) != 0) {
+				rprintf(FERROR,"pop_dir %s : %s\n",
+					dir,strerror(errno));
+				exit_cleanup(RERR_FILESELECT);
+			}
+		}
 	}
 
 	if (f != -1) {
@@ -680,6 +729,7 @@ struct file_list *recv_file_list(int f)
   struct file_list *flist;
   unsigned char flags;
   int64 start_read;
+  extern int list_only;
 
   if (verbose && recurse && !am_server) {
     rprintf(FINFO,"receiving file list ... ");
@@ -746,6 +796,14 @@ struct file_list *recv_file_list(int f)
 	  io_error |= read_int(f);
   }
 
+  if (list_only) {
+	  int i;
+	  for (i=0;i<flist->count;i++) {
+		  list_file_entry(flist->files[i]);
+	  }
+  }
+
+
   if (verbose > 2)
     rprintf(FINFO,"recv_file_list done\n");
 
@@ -876,10 +934,10 @@ static void clean_flist(struct file_list *flist, int strip_root)
 	for (i=0;i<flist->count;i++) {
 		rprintf(FINFO,"[%d] i=%d %s %s mode=0%o len=%d\n",
 			getpid(), i, 
-			flist->files[i]->dirname,
-			flist->files[i]->basename,
+			NS(flist->files[i]->dirname),
+			NS(flist->files[i]->basename),
 			flist->files[i]->mode,
-			flist->files[i]->length);
+			(int)flist->files[i]->length);
 	}
 }
 
@@ -898,7 +956,9 @@ char *f_name(struct file_struct *f)
 	n = (n+1)%10;
 
 	if (f->dirname) {
-		slprintf(p, MAXPATHLEN-1, "%s/%s", f->dirname, f->basename);
+		strlcpy(p, f->dirname, MAXPATHLEN-1);
+		strlcat(p, "/", MAXPATHLEN-1);
+		strlcat(p, f->basename, MAXPATHLEN-1);
 	} else {
 		strlcpy(p, f->basename, MAXPATHLEN-1);
 	}
