@@ -36,10 +36,7 @@ static void report(int f)
 	extern int do_stats;
 
 	if (am_daemon) {
-		syslog(LOG_INFO,"wrote %.0f bytes  read %.0f bytes  total size %.0f\n",
-		       (double)stats.total_written,
-		       (double)stats.total_read,
-		       (double)stats.total_size);
+		log_exit(0);
 		if (f == -1 || !am_sender) return;
 	}
 
@@ -181,13 +178,13 @@ static char *get_local_name(struct file_list *flist,char *name)
 			if (!push_dir(name, 0)) {
 				rprintf(FERROR,"push_dir %s : %s (1)\n",
 					name,strerror(errno));
-				exit_cleanup(1);
+				exit_cleanup(RERR_FILESELECT);
 			}
 			return NULL;
 		}
 		if (flist->count > 1) {
 			rprintf(FERROR,"ERROR: destination must be a directory when copying more than 1 file\n");
-			exit_cleanup(1);
+			exit_cleanup(RERR_FILESELECT);
 		}
 		return name;
 	}
@@ -200,7 +197,7 @@ static char *get_local_name(struct file_list *flist,char *name)
 
 	if (do_mkdir(name,0777 & ~orig_umask) != 0) {
 		rprintf(FERROR,"mkdir %s : %s (1)\n",name,strerror(errno));
-		exit_cleanup(1);
+		exit_cleanup(RERR_FILEIO);
 	} else {
 		if (verbose > 0)
 			rprintf(FINFO,"created directory %s\n",name);
@@ -209,7 +206,7 @@ static char *get_local_name(struct file_list *flist,char *name)
 	if (!push_dir(name, 0)) {
 		rprintf(FERROR,"push_dir %s : %s (2)\n",
 			name,strerror(errno));
-		exit_cleanup(1);
+		exit_cleanup(RERR_FILESELECT);
 	}
 
 	return NULL;
@@ -231,7 +228,7 @@ static void do_server_sender(int f_in, int f_out, int argc,char *argv[])
   
 	if (!relative_paths && !push_dir(dir, 0)) {
 		rprintf(FERROR,"push_dir %s: %s (3)\n",dir,strerror(errno));
-		exit_cleanup(1);
+		exit_cleanup(RERR_FILESELECT);
 	}
 	argc--;
 	argv++;
@@ -278,7 +275,7 @@ static int do_recv(int f_in,int f_out,struct file_list *flist,char *local_name)
 
 	if (pipe(recv_pipe) < 0) {
 		rprintf(FERROR,"pipe failed in do_recv\n");
-		exit_cleanup(1);
+		exit_cleanup(RERR_SOCKETIO);
 	}
   
 	io_flush();
@@ -333,7 +330,7 @@ static void do_server_recv(int f_in, int f_out, int argc,char *argv[])
 		if (!am_daemon && !push_dir(dir, 0)) {
 			rprintf(FERROR,"push_dir %s : %s (4)\n",
 				dir,strerror(errno));
-			exit_cleanup(1);
+			exit_cleanup(RERR_FILESELECT);
 		}    
 	}
 
@@ -343,7 +340,7 @@ static void do_server_recv(int f_in, int f_out, int argc,char *argv[])
 	flist = recv_file_list(f_in);
 	if (!flist || flist->count == 0) {
 		rprintf(FERROR,"server_recv: nothing to do\n");
-		exit_cleanup(1);
+		exit_cleanup(RERR_FILESELECT);
 	}
 	
 	if (argc > 0) {    
@@ -387,6 +384,7 @@ int client_run(int f_in, int f_out, int pid, int argc, char *argv[])
 	int status = 0, status2 = 0;
 	char *local_name = NULL;
 	extern int am_sender;
+	extern int list_only;
 
 	setup_protocol(f_out,f_in);
 	
@@ -415,6 +413,8 @@ int client_run(int f_in, int f_out, int pid, int argc, char *argv[])
 		report(-1);
 		exit_cleanup(status);
 	}
+
+	if (argc == 0) list_only = 1;
 	
 	send_exclude_list(f_out);
 	
@@ -464,6 +464,26 @@ static int start_client(int argc, char *argv[])
 	extern int local_server;
 	extern int am_sender;
 	extern char *shell_cmd;
+	extern int rsync_port;
+
+	if (strncasecmp(URL_PREFIX, argv[0], strlen(URL_PREFIX)) == 0) {
+		char *host, *path;
+
+		host = argv[0] + strlen(URL_PREFIX);
+		p = strchr(host,'/');
+		if (p) {
+			*p = 0;
+			path = p+1;
+		} else {
+			path="";
+		}
+		p = strchr(host,':');
+		if (p) {
+			rsync_port = atoi(p+1);
+			*p = 0;
+		}
+		return start_socket_client(host, path, argc-1, argv+1);
+	}
 
 	p = find_colon(argv[0]);
 
@@ -473,9 +493,9 @@ static int start_client(int argc, char *argv[])
 			return start_socket_client(argv[0], p+2, argc-1, argv+1);
 		}
 
-		if (argc < 2) {
+		if (argc < 1) {
 			usage(FERROR);
-			exit_cleanup(1);
+			exit_cleanup(RERR_SYNTAX);
 		}
 
 		am_sender = 0;
@@ -497,7 +517,7 @@ static int start_client(int argc, char *argv[])
 
 		if (argc < 2) {
 			usage(FERROR);
-			exit_cleanup(1);
+			exit_cleanup(RERR_SYNTAX);
 		}
 		
 		if (local_server) {
@@ -528,9 +548,9 @@ static int start_client(int argc, char *argv[])
 			shell_path?shell_path:"");
 	}
 	
-	if (!am_sender && argc != 1) {
+	if (!am_sender && argc > 1) {
 		usage(FERROR);
-		exit_cleanup(1);
+		exit_cleanup(RERR_SYNTAX);
 	}
 	
 	pid = do_cmd(shell_cmd,shell_machine,shell_user,shell_path,&f_in,&f_out);
@@ -545,7 +565,7 @@ static int start_client(int argc, char *argv[])
 
 
 static RETSIGTYPE sigusr1_handler(int val) {
-	exit_cleanup(1);
+	exit_cleanup(RERR_SIGNAL);
 }
 
 int main(int argc,char *argv[])
@@ -565,7 +585,7 @@ int main(int argc,char *argv[])
 
 	if (argc < 2) {
 		usage(FERROR);
-		exit_cleanup(1);
+		exit_cleanup(RERR_SYNTAX);
 	}
 
 	/* we set a 0 umask so that correct file permissions can be
@@ -573,7 +593,7 @@ int main(int argc,char *argv[])
 	orig_umask = (int)umask(0);
 
 	if (!parse_arguments(argc, argv)) {
-		exit_cleanup(1);
+		exit_cleanup(RERR_SYNTAX);
 	}
 
 	argc -= optind;
@@ -584,6 +604,7 @@ int main(int argc,char *argv[])
 	signal(SIGINT,SIGNAL_CAST sig_int);
 	signal(SIGPIPE,SIGNAL_CAST sig_int);
 	signal(SIGHUP,SIGNAL_CAST sig_int);
+	signal(SIGTERM,SIGNAL_CAST sig_int);
 
 	if (am_daemon) {
 		return daemon_main();
@@ -591,7 +612,7 @@ int main(int argc,char *argv[])
 
 	if (argc < 1) {
 		usage(FERROR);
-		exit_cleanup(1);
+		exit_cleanup(RERR_SYNTAX);
 	}
 
 	if (dry_run)
@@ -600,7 +621,7 @@ int main(int argc,char *argv[])
 #ifndef SUPPORT_LINKS
 	if (!am_server && preserve_links) {
 		rprintf(FERROR,"ERROR: symbolic links not supported\n");
-		exit_cleanup(1);
+		exit_cleanup(RERR_UNSUPPORTED);
 	}
 #endif
 
