@@ -91,6 +91,8 @@ void send_files(struct file_list *flist,int f_out,int f_in)
 	int i;
 	struct file_struct *file;
 	int phase = 0;
+	extern struct stats stats;		
+	struct stats initial_stats;
 
 	if (verbose > 2)
 		rprintf(FINFO,"send_files starting\n");
@@ -116,7 +118,7 @@ void send_files(struct file_list *flist,int f_out,int f_in)
 		if (i < 0 || i >= flist->count) {
 			rprintf(FERROR,"Invalid file index %d (count=%d)\n", 
 				i, flist->count);
-			exit_cleanup(1);
+			exit_cleanup(RERR_PROTOCOL);
 		}
 
 		file = flist->files[i];
@@ -142,12 +144,15 @@ void send_files(struct file_list *flist,int f_out,int f_in)
 			rprintf(FINFO,"send_files(%d,%s)\n",i,fname);
 	  
 		if (dry_run) {	
-			if (!am_server && verbose)
-				rprintf(FINFO,"%s\n",fname);
+			if (!am_server) {
+				log_transfer(file, fname+offset);
+			}
 			write_int(f_out,i);
 			continue;
 		}
 
+		initial_stats = stats;
+
 		s = receive_sums(f_in);
 		if (!s) {
 			io_error = 1;
@@ -182,7 +187,7 @@ void send_files(struct file_list *flist,int f_out,int f_in)
 		if (verbose > 2)
 			rprintf(FINFO,"send_files mapped %s of size %d\n",
 				fname,(int)st.st_size);
-	  
+
 		write_int(f_out,i);
 	  
 		write_int(f_out,s->count);
@@ -192,11 +197,14 @@ void send_files(struct file_list *flist,int f_out,int f_in)
 		if (verbose > 2)
 			rprintf(FINFO,"calling match_sums %s\n",fname);
 	  
-		if (!am_server && verbose)
-			rprintf(FINFO,"%s\n",fname+offset);
+		if (!am_server) {
+			log_transfer(file, fname+offset);
+		}
 	  
 		match_sums(f_out,s,buf,st.st_size);
-	  
+
+		log_send(file, &initial_stats);
+	  	  
 		if (buf) unmap_file(buf);
 		close(fd);
 	  
