@@ -125,12 +125,12 @@ void start_accept_loop(int port, int (*fn)(int ))
 	/* open an incoming socket */
 	s = open_socket_in(SOCK_STREAM, port);
 	if (s == -1)
-		exit_cleanup(1);
+		exit_cleanup(RERR_SOCKETIO);
 
 	/* ready to listen */
 	if (listen(s, 5) == -1) {
 		close(s);
-		exit_cleanup(1);
+		exit_cleanup(RERR_SOCKETIO);
 	}
 
 
@@ -320,13 +320,17 @@ char *client_addr(int fd)
 	struct sockaddr_in *sockin = (struct sockaddr_in *) (&sa);
 	int     length = sizeof(sa);
 	static char addr_buf[100];
+	static int initialised;
+
+	if (initialised) return addr_buf;
+
+	initialised = 1;
 
 	if (getpeername(fd, &sa, &length)) {
-		exit_cleanup(1);
+		exit_cleanup(RERR_SOCKETIO);
 	}
-
+	
 	strlcpy(addr_buf,(char *)inet_ntoa(sockin->sin_addr), sizeof(addr_buf)-1);
-
 	return addr_buf;
 }
 
@@ -343,11 +347,16 @@ char *client_name(int fd)
 	struct hostent *hp;
 	char **p;
 	char *def = "UNKNOWN";
+	static int initialised;
+
+	if (initialised) return name_buf;
+
+	initialised = 1;
 
 	strcpy(name_buf,def);
 
 	if (getpeername(fd, &sa, &length)) {
-		exit_cleanup(1);
+		exit_cleanup(RERR_SOCKETIO);
 	}
 
 	/* Look up the remote host name. */
